/*------------------------------------------------------------------------------
	>	SharePoint Field Name Normalization/Denormalization Example	
		------------------------------------------------------------------------
		The MIT License

		Copyright (c) 2007 Charles Chen (cchen@charliedigital.com)

		Permission is hereby granted, free of charge, to any person obtaining a copy
		of this software and associated documentation files (the "Software"), to deal
		in the Software without restriction, including without limitation the rights
		to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
		copies of the Software, and to permit persons to whom the Software is
		furnished to do so, subject to the following conditions:

		The above copyright notice and this permission notice shall be included in
		all copies or substantial portions of the Software.

		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
		IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
		FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
		AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
		LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
		OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
		THE SOFTWARE.
		------------------------------------------------------------------------
		http://www.charliedigital.com
------------------------------------------------------------------------------*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace SharePointNormalizationConsole {
	
    class Program {
        private static readonly Regex specialCharactersPattern 
            = new Regex("[\\[*($%&)<>!?\\/\"{}\\s+-='@~`#\\\\:;^\\]]", RegexOptions.Compiled);

        private static readonly Regex encodedCharactersPattern
            = new Regex("_x00(\\d{2})_", RegexOptions.Compiled);

        static void Main(string[] args) {
            Program program = new Program();
            program.Run();
        }

        public void Run() {
            bool stop = false;

            while(!stop) {
                Console.Write("Enter static name:");
                string key = Console.ReadLine();

                if(key == "Q" || key == "q" || string.IsNullOrEmpty(key)) {
                    stop = true;
                    Console.WriteLine("\n\nExiting.");
                }
                else {
                    string normalizedKey = NormalizeSharePointPropertyName(key);

                    Console.WriteLine("{0} == {1}", key, normalizedKey);
                    Console.WriteLine("{0} == {1}", normalizedKey, DenormalizeSharePointPropertyName(key));
                    Console.Out.WriteLine("__________________________________");
                }
            }
        }

        /// <summary>
        /// Normalizes the name of the SharePoint property name.
        /// </summary>
        /// <param name="key">The "human readable" key/property name.</param>
        /// <returns>
        /// The string with hex representation in place of special ASCII 
        /// characters.
        /// </returns>
        private static string NormalizeSharePointPropertyName(string key) {
            return specialCharactersPattern.Replace(key, ReplaceSpecialCharacter);
        }

        /// <summary>
        /// Custom match evaluator to replace special characters within the input 
        /// string.
        /// </summary>
        /// <param name="match">The pattern match.</param>
        /// <returns>The formatted version of the string.</returns>
        private static string ReplaceSpecialCharacter(Match match) {
            string replacement = string.Format("_x00{0}_",
                Uri.HexEscape(match.Value[0]).TrimStart('%'));

            return replacement;
        }

        /// <summary>
        /// Denormalizes the name of the SharePoint property.
        /// </summary>
        /// <param name="key">The normalized key/property name (static name).</param>
        /// <returns>
        /// The denormalized form of the input string ("human readable" 
        /// with hex patterns replaced).
        /// </returns>
        private static string DenormalizeSharePointPropertyName(string key) {
            return encodedCharactersPattern.Replace(key, DecodeSpecialCharacter);
        }

        /// <summary>
        /// Custom match evaluator to replace the hex characters with special characters.
        /// </summary>
        /// <param name="match">The pattern match.</param>
        /// <returns>The ASCII format of the special character encoded in hex.</returns>
        private static string DecodeSpecialCharacter(Match match) {
            int start = 0;

            string replacement =
                Convert.ToString(
                    Uri.HexUnescape(Convert.ToString(match.Value[0]), ref start)
                );

            return replacement;
        }
    }
}
